## $Id: pdf.py 11887 2014-10-28 17:48:22Z uli $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Reusable components for pdf generation.
"""

import os
from string import Template
from waeup.kofa.browser.pdf import PDFCreator

class CustomPDFCreator(PDFCreator):
    """A utility to help with generating PDF docs.
    """

    header_logo_path = os.path.join(
        os.path.dirname(__file__), 'static', 'pdf_logo.png')
    #watermark_path = os.path.join(
    #    os.path.dirname(__file__), 'static', 'pdf_watermark.jpg')

    #watermark_pos = [-25, 100]
    logo_pos = [507, 740, 70]

    signature_img_path = os.path.join(
        os.path.dirname(__file__), 'static', 'pdf_signature.png')

    def substitute_markers(self, text):
        """If `text` contains some marker, substitute it.

        Markers are standard library string template placeholders. See
        https://docs.python.org/2/library/string.html and
        `string.Template` for details. Roughly, placeholders look like
        ``${some_marker}``.

        Valid markers:
           ``signature_img_path`` -- path to signature image stored in
                                     `signature_img_path`` attribute.
        """
        text = Template(text)
        return text.safe_substitute(
            {
                'signature_img_path': self.signature_img_path,
                }
            )

    def create_pdf(self, *args, **kw):
        """Create PDF from args and keywords.

        Unlike the base method, we mangle any `notes` keyword and
        substitute any valid placeholder strings in it. See
        `substitute_markers` for a list of valid placeholders.

        XXX: This is only a quick, dirty hack, that should not appear
             in this package. Instead it should be moved to the
             waeup.kofa basepackage.

        Please note: string substitution like this can introduce nasty
        security holes. For instance all dynamic input in `notes` must
        be cleaned up and placeholder expressions should be removed
        from it before using it in `note`.

        Also, `note` is clearly misused in this case. We might want to
        look for a more general approach, maybe z3c.rml?
        """
        if 'note' in kw.keys() and kw['note'] is not None:
            kw['note'] = self.substitute_markers(kw['note'])
        return super(CustomPDFCreator, self).create_pdf(*args, **kw)
